jest.mock("@braintree/asset-loader", () => {
  return {
    loadScript: jest.fn(),
    loadStylesheet: jest.fn(),
  };
});

import { constants } from "../constants";
import assetLoader from "@braintree/asset-loader";
import loadAxo from "../loadAxoScript";
import { AxoSupportedPlatforms } from "../types";

describe("loadAxoScript", () => {
  let testContext;
  const version = "3.96";

  beforeEach(() => {
    testContext = {};
    testContext.mockLoadResponse = {
      metadata: {
        localeUrl: "https://www.paypalobjects.com/connect-boba/locales/",
      },
    };
    testContext.client = {
      getVersion: () => {
        return version;
      },
    };
  });

  it("should call loadScript with the minified URL when specified", async () => {
    const promise = loadAxo({
      platform: AxoSupportedPlatforms.BT,
      btSdkVersion: version,
      minified: true,
    });

    expect(promise).resolves.toStrictEqual(testContext.mockLoadResponse);
    expect(assetLoader.loadScript).toHaveBeenCalled();
    expect(assetLoader.loadScript).toHaveBeenCalledWith(
      expect.objectContaining({
        src: `${constants.CDNX_PROD}/${constants.AXO_URL.minified}`,
      })
    );
  });

  it("should load the unminified URL when specified", () => {
    const promise = loadAxo({
      platform: AxoSupportedPlatforms.BT,
      btSdkVersion: version,
      minified: false,
    });

    expect(promise).resolves.toStrictEqual(testContext.mockLoadResponse);
    expect(assetLoader.loadScript).toHaveBeenCalled();
    expect(assetLoader.loadScript).toHaveBeenCalledWith(
      expect.objectContaining({
        src: `${constants.CDNX_PROD}/${constants.AXO_URL.unminified}`,
      })
    );
  });

  it("should default to the minified script when no environment is specified", () => {
    const promise = loadAxo({
      platform: AxoSupportedPlatforms.BT,
      btSdkVersion: version,
    });
    expect(promise).resolves.toStrictEqual(testContext.mockLoadResponse);
    expect(assetLoader.loadScript).toHaveBeenCalled();
    expect(assetLoader.loadScript).toHaveBeenCalledWith(
      expect.objectContaining({
        src: `${constants.CDNX_PROD}/${constants.AXO_URL.minified}`,
      })
    );
  });

  it("should load the bundle override URL when specified", () => {
    const bundleId = "2bc4343c92655";
    const promise = loadAxo({
      platform: AxoSupportedPlatforms.BT,
      btSdkVersion: version,
      metadata: { bundleIdOverride: bundleId },
    });

    expect(promise).resolves.toStrictEqual({
      metadata: {
        localeUrl: `https://cdn-${bundleId}.static.engineering.dev.paypalinc.com/${constants.LOCALE_PATH}`,
      },
    });
    expect(assetLoader.loadScript).toHaveBeenCalled();
    expect(assetLoader.loadScript).toHaveBeenCalledWith(
      expect.objectContaining({
        src: `https://cdn-${bundleId}.static.engineering.dev.paypalinc.com/${constants.AXO_URL.minified}`,
      })
    );
  });

  it("should call loadScript with the minified URL and bundle override URL when specified", async () => {
    const bundleId = "9df9369c92655";
    const promise = loadAxo({
      platform: AxoSupportedPlatforms.BT,
      btSdkVersion: version,
      minified: true,
      metadata: { bundleIdOverride: bundleId },
    });
    expect(promise).resolves.toStrictEqual({
      metadata: {
        localeUrl: `https://cdn-${bundleId}.static.engineering.dev.paypalinc.com/${constants.LOCALE_PATH}`,
      },
    });
    expect(assetLoader.loadScript).toHaveBeenCalled();
    expect(assetLoader.loadScript).toHaveBeenCalledWith(
      expect.objectContaining({
        src: `https://cdn-${bundleId}.static.engineering.dev.paypalinc.com/${constants.AXO_URL.minified}`,
      })
    );
  });

  it("should load Bt hcf and axo when platform is bt", () => {
    const promise = loadAxo({
      platform: AxoSupportedPlatforms.BT,
      btSdkVersion: version,
      minified: true,
    });

    expect(promise).resolves.toStrictEqual({
      metadata: {
        localeUrl: `${constants.CDNX_PROD}/${constants.LOCALE_PATH}`,
      },
    });
    expect(assetLoader.loadScript).toHaveBeenCalledTimes(2);
    expect(assetLoader.loadScript).toHaveBeenCalledWith(
      expect.objectContaining({
        id: `hcf-${version}`,
        src: `https://js.braintreegateway.com/web/${version}/js/hosted-fields.min.js`,
      })
    );
    expect(assetLoader.loadScript).toHaveBeenCalledWith(
      expect.objectContaining({
        src: `${constants.CDNX_PROD}/${constants.AXO_URL.minified}`,
      })
    );
  });

  it("should load Bt client, hcf and Axo when platform is ppcp", () => {
    const promise = loadAxo({
      platform: AxoSupportedPlatforms.PPCP,
      btSdkVersion: version,
      minified: true,
    });

    expect(promise).resolves.toStrictEqual({
      metadata: {
        localeUrl: `${constants.CDNX_PROD}/${constants.LOCALE_PATH}`,
      },
    });

    expect(assetLoader.loadScript).toHaveBeenCalledTimes(3);

    expect(assetLoader.loadScript).toHaveBeenCalledWith(
      expect.objectContaining({
        id: `client-${version}`,
        src: `https://js.braintreegateway.com/web/${version}/js/client.min.js`,
      })
    );
    expect(assetLoader.loadScript).toHaveBeenCalledWith(
      expect.objectContaining({
        id: `hcf-${version}`,
        src: `https://js.braintreegateway.com/web/${version}/js/hosted-fields.min.js`,
      })
    );
    expect(assetLoader.loadScript).toHaveBeenCalledWith(
      expect.objectContaining({
        src: `${constants.CDNX_PROD}/${constants.AXO_URL.minified}`,
      })
    );
  });
});
