import { loadScript } from "@braintree/asset-loader";
import { constants } from "./constants";
import {
  LoadAxoOptions,
  GenerateAssetUrl,
  AxoSupportedPlatforms,
} from "./types";
import { safeLoadBtModule } from "./lib/safeLoadBtModule";
import { btModulesLoadConfig } from "./lib/safeLoadBtModule/types";

/**
 * Loads accelerated checkout components.
 * @param options object with a minified parameter to determine if the script that is loaded should be minified or not (defaults to true if)
 * @returns an object with metadata with a localeUrl parameter to be read by AXO SDK
 */
async function loadAxo(options: LoadAxoOptions) {
  try {
    const { btSdkVersion } = options;
    const assetUrl = getAssetsUrl(options);
    const localeUrl = getLocaleUrl(options);

    if (options.platform === AxoSupportedPlatforms.BT) {
      await Promise.all([
        loadBtHcfScript(btSdkVersion),
        loadAXOScript(assetUrl),
      ]);
    } else if (options.platform === AxoSupportedPlatforms.PPCP) {
      await Promise.all([
        loadBtClientScript(btSdkVersion),
        loadBtHcfScript(btSdkVersion),
        loadAXOScript(assetUrl),
      ]);
    } else {
      throw new Error("unsupported axo platform");
    }

    return { metadata: { localeUrl } };
  } catch (err) {
    return Promise.reject(err);
  }
}

/**
 * Reads the version and to load the correct version of HCF, so we don't run into a version mismatch between hcf and client
 * @param version (Required) string version that should be passed from the client getVersion
 * @returns Promise<HTMLScriptElement>
 */
function loadBtHcfScript(version: string) {
  return safeLoadBtModule(btModulesLoadConfig.hostedFields, version);
}

/**
 * Reads the version and to load the correct version of BT client, so we don't run into a version mismatch between client and other modules
 * @param version (Required) string version that should be passed from the client getVersion
 * @returns Promise<true> when
 * @returns Promise<HTMLScriptElement>
 */
function loadBtClientScript(version: string) {
  return safeLoadBtModule(btModulesLoadConfig.client, version);
}

/**
 * Reads the url and to load the axo bundle script
 * @param url (Required) string url for the correct axo asset
 * @returns Promise<HTMLScriptElement>
 */
function loadAXOScript(url: string) {
  return loadScript({
    id: "axo-id",
    src: url,
    forceScriptReload: true,
  });
}

/**
 * Prepends the domain to the asset url
 * @param options object with assetUrl and bundleid parameters to determine which URL to return
 * @returns full domain and assets URL as string
 */
function generateAssetUrl({ assetUrl, bundleId }: GenerateAssetUrl): string {
  return bundleId
    ? `https://cdn-${bundleId}.static.engineering.dev.paypalinc.com/${assetUrl}`
    : `${constants.CDNX_PROD}/${assetUrl}`;
}

/**
 * Retrieves either the minified or unminified assets URL as specified
 * @param options (Optional) object with a minified and metadata with bundleIdOverride parameters to determine which URL to return
 * @returns assets URL as string
 */
function getAssetsUrl(options?: LoadAxoOptions): string {
  const assetUrl =
    options?.minified !== false
      ? constants.AXO_URL.minified
      : constants.AXO_URL.unminified;

  return generateAssetUrl({
    assetUrl,
    bundleId: options?.metadata?.bundleIdOverride,
  });
}

/**
 * Retrieves the Locales URL, the path to our language files
 * @param options (Optional) object with a minified and metadata with bundleIdOverride parameters to determine which URL to return
 * @returns locale URL as string
 */
function getLocaleUrl(options?: LoadAxoOptions): string {
  return generateAssetUrl({
    assetUrl: constants.LOCALE_PATH,
    bundleId: options?.metadata?.bundleIdOverride,
  });
}

export default loadAxo;
